"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderArrayFieldComponent = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const codegen_ui_1 = require("@aws-amplify/codegen-ui");
const typescript_1 = require("typescript");
const form_state_1 = require("./form-state");
const event_handler_props_1 = require("./event-handler-props");
const render_checkers_1 = require("./render-checkers");
const model_values_1 = require("./model-values");
const invalid_variable_helpers_1 = require("./invalid-variable-helpers");
function getOnChangeAttribute({ setStateName, fieldName, renderedFieldName, fieldConfigs, isLimitedToOneValue, }) {
    const fieldConfig = fieldConfigs[fieldName];
    const { dataType, componentType } = fieldConfig;
    let valueName = typescript_1.factory.createIdentifier('values');
    const onChangeArgName = typescript_1.factory.createIdentifier('items');
    let argToValue = onChangeArgName;
    if (isLimitedToOneValue) {
        argToValue = typescript_1.factory.createElementAccessExpression(onChangeArgName, typescript_1.factory.createNumericLiteral('0'));
        valueName = typescript_1.factory.createIdentifier('value');
    }
    const setStateStatements = [
        typescript_1.factory.createExpressionStatement((0, form_state_1.setFieldState)(renderedFieldName, valueName)),
        typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(setStateName, undefined, [
            (0, form_state_1.getDefaultValueExpression)(fieldName, componentType, dataType),
        ])),
    ];
    if ((0, render_checkers_1.isModelDataType)(fieldConfig)) {
        setStateStatements.push(typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createIdentifier(`set${(0, form_state_1.capitalizeFirstLetter)((0, form_state_1.getCurrentDisplayValueName)(renderedFieldName))}`), undefined, [(0, form_state_1.getDefaultValueExpression)(fieldName, componentType, dataType, false, true)])));
    }
    return typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('onChange'), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createArrowFunction([typescript_1.factory.createModifier(typescript_1.SyntaxKind.AsyncKeyword)], undefined, [typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, onChangeArgName, undefined, undefined)], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([typescript_1.factory.createVariableDeclaration(valueName, undefined, undefined, argToValue)], typescript_1.NodeFlags.Let)),
        (0, event_handler_props_1.buildOverrideOnChangeStatement)(fieldName, fieldConfigs, valueName),
        ...setStateStatements,
    ], true))));
}
/*
  <ArrayField
    onChange = { async(items) => {
      setBreeds(items);
      setCurrentBreedsValue('');
    }}
    currentBreedsValue = { currentBreedsValue }
    hasError = { errors.breeds?.hasError }
    setFieldValue = { setCurrentBreedsValue }
    inputFieldRef={ breedsRef }
    >
    <ExampleInputField />
  </ArrayField>

  wraps input field component with array field component
 */
const renderArrayFieldComponent = (fieldName, fieldLabel, fieldConfigs, inputField) => {
    const fieldConfig = fieldConfigs[fieldName];
    const { sanitizedFieldName, dataType, componentType } = fieldConfig;
    const isLimitedToOneValue = fieldConfig.relationship &&
        (fieldConfig.relationship.type === 'HAS_ONE' || fieldConfig.relationship.type === 'BELONGS_TO');
    const renderedFieldName = sanitizedFieldName || fieldName;
    const stateName = (0, form_state_1.getCurrentValueIdentifier)(renderedFieldName);
    const setStateName = (0, form_state_1.getSetNameIdentifier)((0, form_state_1.getCurrentValueName)(renderedFieldName));
    const props = [];
    // render `?? []` if nested.
    let itemsExpression = renderedFieldName.includes('.')
        ? typescript_1.factory.createBinaryExpression(typescript_1.factory.createIdentifier(renderedFieldName), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionQuestionToken), typescript_1.factory.createArrayLiteralExpression([], false))
        : typescript_1.factory.createIdentifier(renderedFieldName);
    if (isLimitedToOneValue) {
        // "book ? [book] : []"
        itemsExpression = typescript_1.factory.createConditionalExpression(typescript_1.factory.createIdentifier(renderedFieldName), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionToken), typescript_1.factory.createArrayLiteralExpression([typescript_1.factory.createIdentifier(renderedFieldName)], false), typescript_1.factory.createToken(typescript_1.SyntaxKind.ColonToken), typescript_1.factory.createArrayLiteralExpression([], false));
        props.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('lengthLimit'), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createNumericLiteral('1'))));
    }
    props.push(getOnChangeAttribute({ fieldName, isLimitedToOneValue, fieldConfigs, renderedFieldName, setStateName }));
    props.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier(`currentFieldValue`), typescript_1.factory.createJsxExpression(undefined, stateName)), typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier(`label`), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createStringLiteral(fieldLabel))));
    props.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('items'), typescript_1.factory.createJsxExpression(undefined, itemsExpression)));
    props.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('hasError'), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createPropertyAccessChain((0, codegen_ui_1.isValidVariableName)(fieldName)
        ? typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('errors'), typescript_1.factory.createIdentifier(fieldName))
        : typescript_1.factory.createElementAccessChain(typescript_1.factory.createIdentifier('errors'), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionDotToken), typescript_1.factory.createStringLiteral(fieldName)), typescript_1.factory.createToken(typescript_1.SyntaxKind.QuestionDotToken), typescript_1.factory.createIdentifier('hasError')))));
    let setFieldValueIdentifier = setStateName;
    if ((0, render_checkers_1.shouldImplementDisplayValueFunction)(fieldConfig)) {
        setFieldValueIdentifier = (0, form_state_1.getSetNameIdentifier)((0, form_state_1.getCurrentDisplayValueName)(renderedFieldName));
        props.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('getBadgeText'), typescript_1.factory.createJsxExpression(undefined, (0, invalid_variable_helpers_1.getElementAccessExpression)(model_values_1.getDisplayValueObjectName, fieldName))));
    }
    /**
    setFieldValue={(model) => {
      setCurrentHasOneUserDisplayValue(getDisplayValue.HasOneUser(model))
      setCurrentHasOneUserValue(model)
    }
     */
    if ((0, render_checkers_1.isModelDataType)(fieldConfig)) {
        const valueArgument = 'model';
        props.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('setFieldValue'), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createArrowFunction(undefined, undefined, [
            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier(valueArgument), undefined, undefined, undefined),
        ], undefined, typescript_1.factory.createToken(typescript_1.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(setFieldValueIdentifier, undefined, [
                typescript_1.factory.createCallExpression((0, invalid_variable_helpers_1.getElementAccessExpression)(model_values_1.getDisplayValueObjectName, fieldName), undefined, [typescript_1.factory.createIdentifier(valueArgument)]),
            ])),
            typescript_1.factory.createExpressionStatement(typescript_1.factory.createCallExpression(setStateName, undefined, [typescript_1.factory.createIdentifier(valueArgument)])),
        ], true)))));
    }
    else {
        props.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('setFieldValue'), typescript_1.factory.createJsxExpression(undefined, setStateName)));
    }
    props.push(typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('inputFieldRef'), typescript_1.factory.createJsxExpression(undefined, typescript_1.factory.createIdentifier((0, form_state_1.getArrayChildRefName)(renderedFieldName)))), typescript_1.factory.createJsxAttribute(typescript_1.factory.createIdentifier('defaultFieldValue'), typescript_1.factory.createJsxExpression(undefined, (0, form_state_1.getDefaultValueExpression)(fieldName, componentType, dataType, false, true))));
    return typescript_1.factory.createJsxElement(typescript_1.factory.createJsxOpeningElement(typescript_1.factory.createIdentifier('ArrayField'), undefined, typescript_1.factory.createJsxAttributes(props)), [inputField], typescript_1.factory.createJsxClosingElement(typescript_1.factory.createIdentifier('ArrayField')));
};
exports.renderArrayFieldComponent = renderArrayFieldComponent;
//# sourceMappingURL=render-array-field.js.map